"""
Copyright (c) 2016-2020 Keith Sterling http://www.keithsterling.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from programy.storage.entities.store import Store


class UserStore(Store):

    def __init__(self):
        Store.__init__(self)

    def add_user(self, userid, clientid):
        raise NotImplementedError("add_user missing from User Store")  # pragma: no cover

    def exists(self, userid, clientid):
        raise NotImplementedError("exists missing from User Store")  # pragma: no cover

    def get_links(self, userid):
        raise NotImplementedError("get_links missing from User Store")  # pragma: no cover

    def remove_user(self, userid, clientid):
        raise NotImplementedError("remove_user missing from User Store")  # pragma: no cover

    def remove_user_from_all_clients(self, userid):
        raise NotImplementedError("remove_user_from_all_clients missing from User Store")  # pragma: no cover
